########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 18/05/2009
# Modified on 	: 19/05/2009 (Creation of real output files, removal of unnecessary code)
#				  20/05/2009 (Change of script names, procedure names and file names to make alternative methods clearer)
#				  20/05/2009 (Adjustment of index positions to account for changes to input files output from other modified scripts)
#				  01/06/2009 (Changed input file name to use input file with lifted positions)
#
#
# Input files 	: K_gene_RepTime_genic.txt (Output from tcl script allocate_RepTime_Kgenes_genic.tcl and stored in Output_Results directory)
#				  K_introns.txt (Output from a combination of tcl scripts INTRONS*.tcl, specifically INTRONS16_Kintron.tcl)
#
# Output files  : K_intron_RepTime_genic.txt (Stored in Output_Files directory)
#
# This scripts allocates average genic replication times to introns located wthin the gene.
# Average genic replication times are extracted from K_gene_RepTime_genic.txt and stored in an array indexed by the ortholog ID of the gene.
# For each intron with substitution rate data, details of its genic replication time together with chromosomal class and number of replication regions 
# assigned to the gene are identifed using the ortholog number to index the array.
# All data for the orthologous intron (K, GC, refseq, positions etc) from K_intron.txt together with the new replication timing data is sent to the new file.
#
########################################################################################################################################################

package provide 06_allocate_RepTime_Kintrons_genic 1.0

proc 06_allocate_RepTime_Kintrons_genic {} {

	puts "-----------------------------------------------------------"
	puts "   ASSIGN GENIC REP TIMES TO INTRONIC SUBSTITUTION RATES   \n"

	# Make variables global
	global contents
	
	# Extract replication times for each chromosome
	puts "Extracting replication times for each ortholog and saving in arrays"

	cd Output_Files
	openfile K_gene_RepTime_genic.txt
	cd ..
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	foreach dataline $contents {
			
		set ortholog [lindex $dataline 0]
		set class [lindex $dataline 21]
		set numberRTregions [lindex $dataline 22]
		set meanRT [lindex $dataline 23]
		set medianRT [lindex $dataline 24]

		set DETAILS($ortholog) "$class\t$numberRTregions\t$meanRT\t$medianRT"
		lappend Orthologs $ortholog
		
		unset ortholog class numberRTregions meanRT medianRT
	}
	unset dataline contents
	
	puts "Replication times for each gene extracted into arrays"
	
	
	
	puts "Identifying genic replication times for each intron"	
	
	# New output file
	set outTXT [open K_intron_RepTime_genic.txt w]
	set outXLS [open K_intron_RepTime_genic.xls w]
	
	puts $outTXT "Ortholog\tIntron_Number\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC\tClass\tNumber_RT_regions\tMean_Replication_Time\tMedian_Replication_Time"
	puts $outXLS "Ortholog\tIntron_Number\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC\tClass\tNumber_RT_regions\tMean_Replication_Time\tMedian_Replication_Time"	
	
	
	# Work through intron data and find replications timing data for the gene the intron is within	
	cd Output_Files
	openfile K_intron_liftOver.txt
	cd ..
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	set count 0
	foreach dataline $contents {
		
		incr count
		if {[expr {$count %100}] == 0} {puts "*** $count of [llength $contents] ***"}

		set ortholog [lindex $dataline 0]
				
		# Only find replication timing details for this intron if the ortholog has replication timing data		
		if {[lsearch $Orthologs $ortholog] >= 0} {
			
			# Send original data and replication timing data (from DETAILS($ortholog) for this intron to the new output file
			puts $outTXT "[lindex $dataline 0]\t[lindex $dataline 1]\t[lindex $dataline 2]\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]\t[lindex $dataline 11]\t[lindex $dataline 12]\t[lindex $dataline 13]\t[lindex $dataline 14]\t[lindex $dataline 15]\t[lindex $dataline 16]\t[lindex $dataline 17]\t[lindex $dataline 18]\t[lindex $dataline 19]\t[lindex $DETAILS($ortholog) 0]\t[lindex $DETAILS($ortholog) 1]\t[lindex $DETAILS($ortholog) 2]\t[lindex $DETAILS($ortholog) 3]"
			puts $outXLS "[lindex $dataline 0]\t[lindex $dataline 1]\t[lindex $dataline 2]\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]\t[lindex $dataline 11]\t[lindex $dataline 12]\t[lindex $dataline 13]\t[lindex $dataline 14]\t[lindex $dataline 15]\t[lindex $dataline 16]\t[lindex $dataline 17]\t[lindex $dataline 18]\t[lindex $dataline 19]\t[lindex $DETAILS($ortholog) 0]\t[lindex $DETAILS($ortholog) 1]\t[lindex $DETAILS($ortholog) 2]\t[lindex $DETAILS($ortholog) 3]"
							
		}
		unset ortholog
		
	} 
	unset dataline contents
	
	# Close ouput files and send them to Output_Files directory
	close $outTXT
	close $outXLS
	unset outTXT outXLS
	
	exec mv K_intron_RepTime_genic.txt Output_Files/K_intron_RepTime_genic.txt
	exec mv K_intron_RepTime_genic.xls Output_Files/K_intron_RepTime_genic.xls
	
	puts "Genic replication times assigned to each intron"
	
	# Unset variables
	unset Orthologs DETAILS

	puts "\nVariables open in 06_allocate_RepTime_Kintrons_genic : [info local]"

return 
}