########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 29/07/2009
# Modified on 	: 30/07/2009 (check correct amount of bases counted by comparison to intron lengths)
# 				  30/07/2009 (Addition of GT skew to output file)
# 				  31/07/2009 (Output of three column file for LDH partial correlation script)
#				  19/08/2009 (Change of output file name)
#				  19/08/2009 (All class results output to full results file but only autosomal to 3 column file)
#
# Input files 	: K_gene_RepTime_genic.txt
#				  (Output by tcl script 04_allocate_RepTime_Kgenes_genic.tcl and stored in Output_Files directory)
#				  IN_$refseqmouse.txt (For every mouse ortholog)
#				  (Output by tcl script INTRONS02_extractintrons.tcl and stored in Intron_Data/INTRONS_Mouse directory)
#		  
# Output files  : K_gene_RepTime_genic_GT.txt (Stored in Output_Files directory)
#				  K_gene_RepTime_genic_GT.xls (Stored in Output_Files directory)
#				  RT_KTK_GT_Autosomes.txt (Stored in Output_Files directory)
#
#
# This script only calculates MOUSE G+T skew
#
# Replication time is known to correlate with expression with highly expressed genes replicationg early and 
# lowly expressed genes replicating late. 
# It is therefore possible that the relationship between substitution rates and replication time is the result, not of 
# late replicating DNA having a higher substitution rate, but of highly expressed (and therefore early replicating DNA) 
# having a lower substitution rate.
# I therefore need to be able to control for expression level and see if the relactionship between substitution rate and
# replication time still holds.
# 
# For each orthologous gene in the data set, the original file in INTRONS_Mouse, containing all the intronic sequences, is identifed.
# This file is opened, the full intronic sequence extracted, and the number of each base counted.
# GT skew is then calculated from the number of each type of base:
#
#		GT skew = ((G + T) - (A + C)) / (G + T + A + C)
#	
# GT skew for EVERY gene is output into the full results file (autosomal, X and Y)
# Only autosomal GT skew, replication time (mean) and KTK are output in a 3 column format for intput into LDH partial correlation script.
#
#
########################################################################################################################################################

package provide 25_GT_skew.tcl

proc 25_GT_skew {} {
	
	puts "-----------------------------------------------------------"
	puts "25        ASSIGN GENIC RT/KTK MEASURES OF G+T SKEW         \n"
	
	# Make variables global
	global contents
	
	# Create new file for results
	set outTXT [open K_gene_RepTime_genic_GT.txt w]
	set outXLS [open K_gene_RepTime_genic_GT.xls w]
	
	puts $outTXT "Ortholog\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tIntrons_Concatenated\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tAlignment_Length\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC\tClass\tNumber_Replication_Regions\tMean_RT\tMedian_RT\tMean_RT_start\tMean_RT_end\tMean_RT_difference\tMedian_RT_start\tMedian_RT_end\tMedian_RT_difference\tA\tT\tG\tC\tN\tGT_SKEW"
	puts $outXLS "Ortholog\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tIntrons_Concatenated\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tAlignment_Length\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC\tClass\tNumber_Replication_Regions\tMean_RT\tMedian_RT\tMean_RT_start\tMean_RT_end\tMean_RT_difference\tMedian_RT_start\tMedian_RT_end\tMedian_RT_difference\tA\tT\tG\tC\tN\tGT_SKEW"

	set out3col [open RT_KTK_GT_Autosomes.txt w]
	puts $out3col "RT_mean\tKTK\tGTskew"


	# Extract full results
	cd Output_Files
	openfile K_gene_RepTime_genic.txt
	cd ..
	set contents1 [split $contents \n]
	set contents1 [lrange $contents1 1 end-1]
	unset contents
	set count 0
	foreach datarow $contents1 {
	
		incr count
		if {[expr {$count %500}] == 0} {puts "Gene $count of [llength $contents1]"}

		set refseqmouse [lindex $datarow 3]

		set seqlength 0
		set basecount 0		
		
		# Initiate base counts for this gene
		foreach base {a t c g n} {set $base 0}; unset base
		
		# Find mouse file
		cd Intron_Data/INTRONS_Mouse
		openfile IN_$refseqmouse.txt
		cd ../..
		
		# Seperate out mouse intronic sequences
		set contents [split $contents >]
		set contents [lrange $contents 1 end]
		foreach intron $contents {
	
			# Extract the sequence & count the bases
			set sequence [split $intron \n]
			set sequence [lindex $sequence 1]
			
			set seqlength [expr {$seqlength + [string length $sequence]}]
		
			for {set position 0} {$position < [string length $sequence]} {incr position} {
				set base [string index $sequence $position]
				if {[regexp {[agct]} $base]} {incr $base} else {incr n}
				unset base
			}; unset position sequence
				
		}; unset intron contents
	
			
		# Calculate G+T skew
		set basecount [expr {$a + $t + $c +$g +$n}]
		set GTskew [expr {[expr {[expr {[format %f $g] + $t}] - [expr {$a + $c}]}] / [expr {$a + $t + $g + $c}]}]

		if {$seqlength != $basecount} {puts "ERROR...Mistake in base counting as not equal to sequence length"; exit}

		# Output GT skew column to full results file
		puts $outTXT "$datarow\t$a\t$t\t$g\t$c\t$n\t$GTskew"
		puts $outXLS "$datarow\t$a\t$t\t$g\t$c\t$n\t$GTskew"
		
		# Only output autosomal GT skew to 3 column file for partial correlation analysis
		if {[lindex $datarow 21] != "Y" | [lindex $datarow 21] != "X"} {
			puts $out3col "[lindex $datarow 21]\t[lindex $datarow 23]\t[lindex $datarow 17]\t$GTskew"
		}

		# Unset variables
		unset refseqmouse
		unset a t c g n
		unset GTskew
		unset basecount seqlength
	
	}; unset datarow contents1
	
	# Close output files and move to Output_Files directory
	close $outTXT
	close $outXLS
	close $out3col
	
	exec mv K_gene_RepTime_genic_GT.txt Output_Files/K_gene_RepTime_genic_GT.txt
	exec mv K_gene_RepTime_genic_GT.xls Output_Files/K_gene_RepTime_genic_GT.xls
	exec mv RT_KTK_GT_Autosomes.txt Output_Files/RT_KTK_GT_Autosomes.txt
	
	puts "\nVariables open in 07_GT_skew : [info local]"
	
return
}
